@@ -1,24 +1,33 @@
 # coding=utf-8
+# Copyright 2018 Sascha Schirra
 #
-# Copyright 2016 Sascha Schirra
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# This file is part of Ropper.
+# 1. Redistributions of source code must retain the above copyright notice, this
+# list of conditions and the following disclaimer.
 #
-# Ropper is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+# this list of conditions and the following disclaimer in the documentation
+# and/or other materials provided with the distribution.
 #
-# Ropper is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# 3. Neither the name of the copyright holder nor the names of its contributors
+# may be used to endorse or promote products derived from this software without
+# specific prior written permission.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" A ND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 from __future__ import print_function
 from filebytes.pe import ImageDirectoryEntry
-from ropper.common.utils import isHex, toHex, getFileNameFromPath
+from ropper.common.utils import isWindows, isHex, toHex, getFileNameFromPath
 from ropper.common.coloredstring import cstr, Color
 from ropper.common.error import RopperError
 from ropper.loaders.loader import Loader, Type
@@ -28,7 +37,7 @@ from ropper.rop import Ropper, Format
 from ropper.gadget import Gadget, GadgetType
 from binascii import unhexlify
 from codecs import encode, decode
-from hashlib import md5
+from ropper.semantic import Analyser, SemanticInformation
 import tempfile
 import re
 import os
@@ -40,6 +49,7 @@ def deleteDuplicates(gadgets, callback=None):
     inst = set()
     count = 0
     added = False
+    len_gadgets = len(gadgets)
     for i,gadget in enumerate(gadgets):
         inst.add(gadget._gadget)
         if len(inst) > count:
@@ -47,7 +57,7 @@ def deleteDuplicates(gadgets, callback=None):
             toReturn.append(gadget)
             added = True
         if callback:
-            callback(gadget, added, float(i)/(len(gadgets)-1))
+            callback(gadget, added, float(i+1)/(len_gadgets))
             added = False
     return toReturn
 
@@ -70,7 +80,7 @@ def filterBadBytes(gadgets, badbytes, callback=None):
     badbytes = formatBadBytes(badbytes)
     if isinstance(gadgets, dict):
         toReturn = {}
-        
+
         gadget_count = 0
         for file, gadget in gadgets.items():
             gadget_count += len(gadget)
@@ -84,8 +94,8 @@ def filterBadBytes(gadgets, badbytes, callback=None):
                     callback(gadget, added, float(i)/(gadget_count-1))
                     added = False
             toReturn[file] = t
-    elif isinstance(gadgets, list): 
-        toReturn = []  
+    elif isinstance(gadgets, list):
+        toReturn = []
         for i, gadget in enumerate(gadgets):
             if not badbytes or not gadget.addressesContainsBytes(badbytes):
                 toReturn.append(gadget)
@@ -98,10 +108,9 @@ def filterBadBytes(gadgets, badbytes, callback=None):
 
 
 def cfgFilterGadgets(binary, gadgets, callback=None):
-    
+
     def intern(gadgets, callback=None,current=0, length=0):
         result = []
-        
         added = False
         for gadget in gadgets:
             loadConfig = binary._binary.dataDirectory[ImageDirectoryEntry.LOAD_CONFIG]
@@ -127,7 +136,7 @@ def cfgFilterGadgets(binary, gadgets, callback=None):
                 added = False
             current += 1
         return result
-    
+
     if isinstance(gadgets,  list):
         gadgetLen = len(gadgets)-1
         return intern(gadgets, callback, length=gadgetLen)
@@ -206,6 +215,12 @@ class Options(object):
         elif cfg_only == None:
             options['cfg_only'] = False
 
+        count_of_findings = options.get('count_of_findings')
+        if count_of_findings != None and not isinstance(count_of_findings, int):
+            raise TypeError('cfg_only has to be an instance of bool')
+        elif count_of_findings == None:
+            options['count_of_findings'] = 5
+
     def items(self):
         for key, value in self.__options_dict.items():
             yield key, value
@@ -236,7 +251,8 @@ class Options(object):
 
 class RopperService(object):
 
-    CACHE_FOLDER = 'ropper_cache'
+    ROPPER_FOLDER = os.path.expanduser('~') + os.path.sep + ".ropper/"
+    CACHE_FOLDER = os.path.expanduser('~') + os.path.sep + ".ropper/cache/"
     CACHE_FILE_COUNT = 16
 
     def __init__(self, options={}, callbacks=None):
@@ -255,7 +271,7 @@ class RopperService(object):
     @property
     def ropper(self):
         return self.__ropper
-    
+
     @property
     def options(self):
         return self.__options
@@ -263,16 +279,16 @@ class RopperService(object):
     @property
     def files(self):
         return list(self.__files)
-    
+
     def __optionChanged(self, option, oldvalue, newvalue):
         if hasattr(self, '_%s_changed' % option):
             func = getattr(self, '_%s_changed' % option)
             func(newvalue)
 
     def __prepareGadgets(self, file, gadgets, type=None):
-        
+
         gadgets = self.__filterBadBytes(gadgets)
-        gadgets = self.__filterCfg(file, gadgets, type) 
+        gadgets = self.__filterCfg(file, gadgets, type)
         if not self.__options.all:
             callback = None
             if self.__callbacks and hasattr(self.__callbacks, '__deleteDoubleGadgetsProgress__'):
@@ -297,25 +313,23 @@ class RopperService(object):
         return gadgets
 
     def __getCacheFileName(self, file):
-        m = md5()
-        m.update(file.loader._binary._bytes)
-        d = m.hexdigest()
-        return "%s_%s_%d_%s_%d_%s" % (getFileNameFromPath(file.name), str(file.arch), self.options.inst_count,str(self.options.type), sys.version_info.major,d)
+        return "%s_%s_%d_%s_%d" % (file.loader.checksum, str(file.arch), self.options.inst_count,str(self.options.type), sys.version_info.major)
 
     def __saveCache(self, file):
+        cache_file = None
         try:
-            temp = tempfile.gettempdir() + os.path.sep + RopperService.CACHE_FOLDER
+            temp = RopperService.CACHE_FOLDER
             if not os.path.exists(temp):
-                os.mkdir(temp)
+                os.makedirs(temp)
 
             cache_file = temp + os.path.sep + self.__getCacheFileName(file)
             count = RopperService.CACHE_FILE_COUNT
-            if len(file.allGadgets) > 1000:
+            if not isWindows() and len(file.allGadgets) > 1000:
                 if os.path.exists(cache_file):
                     os.remove(cache_file)
 
                 length = len(file.allGadgets)
-                
+
                 step = int(length / count)
                 for i in range(count-1):
                     gadgets = file.allGadgets[i*step: (i+1)*step]
@@ -330,12 +344,15 @@ class RopperService(object):
             with open(cache_file,'wb') as f:
                 f.write(encode(repr(file.allGadgets).encode('ascii'),'zip'))
         except BaseException as e:
-            for i in range(1, RopperService.CACHE_FILE_COUNT+1):
-                if os.path.exists(cache_file+'_%d' % i):
-                    os.remove(cache_file+'_%d' % i)
+            print(e)
+            if cache_file:
+                for i in range(1, RopperService.CACHE_FILE_COUNT+1):
+                    if os.path.exists(cache_file+'_%d' % i):
+                        os.remove(cache_file+'_%d' % i)
 
 
     def __loadCachePerProcess(self, fqueue, gqueue):
+        nan=0
         while True:
             cacheFileName = fqueue.get()
             if cacheFileName is None:
@@ -352,18 +369,22 @@ class RopperService(object):
 
     def __loadCache(self, file):
         mp = False
+        nan= 0
         processes = []
         single = False
+        cache_file = None
         try:
-            temp = tempfile.gettempdir() + os.path.sep + RopperService.CACHE_FOLDER
+            temp = RopperService.CACHE_FOLDER
             cache_file = temp + os.path.sep + self.__getCacheFileName(file)
 
             if not os.path.exists(cache_file):
                 if not os.path.exists(cache_file+'_%d' % 1):
                     return
                 else:
+                    if isWindows():
+                        raise RopperError('Cache has to be cleared.')
                     mp = True and multiprocessing.cpu_count()>1
-            else: 
+            else:
                 single = True
             if self.__callbacks and hasattr(self.__callbacks, '__message__'):
                 self.__callbacks.__message__('Load gadgets from cache')
@@ -375,6 +396,8 @@ class RopperService(object):
                     with open(cache_file,'rb') as f:
                         data = f.read()
                         all_gadgets.extend(eval(decode(data,'zip')))
+                        if self.__callbacks and hasattr(self.__callbacks, '__gadgetSearchProgress__'):
+                            self.__callbacks.__gadgetSearchProgress__(None, all_gadgets, 1.0)
                 else:
                     for i in range(1,RopperService.CACHE_FILE_COUNT+1):
                         if os.path.exists(cache_file+'_%d' % i):
@@ -384,7 +407,7 @@ class RopperService(object):
                                 if self.__callbacks and hasattr(self.__callbacks, '__gadgetSearchProgress__'):
                                     self.__callbacks.__gadgetSearchProgress__(None, all_gadgets, float(i)/RopperService.CACHE_FILE_COUNT)
                 return all_gadgets
-                
+
             else:
                 count = min(multiprocessing.cpu_count(),RopperService.CACHE_FILE_COUNT)
 
@@ -410,19 +433,18 @@ class RopperService(object):
                 return sorted(all_gadgets, key=Gadget.simpleInstructionString)
         except KeyboardInterrupt:
             if mp:
-                for i in range(count):
-                    p = processes[i]
+                for p in processes:
                     if p and p.is_alive():
                         p.terminate()
         except BaseException as e:
             if mp:
-                for i in range(count):
-                    p = processes[i]
+                for p in processes:
                     if p and p.is_alive():
                         p.terminate()
-            for i in range(1,RopperService.CACHE_FILE_COUNT+1):
-                if os.path.exists(cache_file+'_%d' % i):
-                    os.remove(cache_file+'_%d' % i)
+            if cache_file:
+                for i in range(1,RopperService.CACHE_FILE_COUNT+1):
+                    if os.path.exists(cache_file+'_%d' % i):
+                        os.remove(cache_file+'_%d' % i)
 
 
     def _badbytes_changed(self, value):
@@ -464,7 +486,7 @@ class RopperService(object):
         return None
 
     def clearCache(self):
-        temp = tempfile.gettempdir() + os.path.sep + RopperService.CACHE_FOLDER
+        temp = RopperService.CACHE_FOLDER
         if os.path.exists(temp):
             import shutil
             shutil.rmtree(temp)
@@ -472,7 +494,7 @@ class RopperService(object):
 
     def getFileFor(self, name):
         return self._getFileFor(name)
-        
+
     def addFile(self, name, bytes=None, arch=None, raw=False):
         if self._getFileFor(name):
             raise RopperError('file is already added: %s' % name)
@@ -481,8 +503,6 @@ class RopperService(object):
             arch=getArchitecture(arch)
 
         loader = Loader.open(name, bytes=bytes, raw=raw, arch=arch)
-        if len(self.__files) > 0 and self.__files[0].loader.arch != loader.arch:
-            raise RopperError('It is not supported to open file with different architectures! Loaded: %s; File to open: %s' % (str(self.__files[0].loader.arch), str(loader.arch)))
         file = FileContainer(loader)
         self.__files.append(file)
 
@@ -560,11 +580,28 @@ class RopperService(object):
 
         return self.__filterBadBytes(to_return)
 
+    def analyseGadgets(self, fileObject):
+        gadgets = fileObject.gadgets
+        analyser = Analyser()
+        cb = None
+        lg = len(gadgets)
+        if self.__callbacks and hasattr(self.__callbacks, '__analyseGadgetsProgress__'):
+            cb = self.__callbacks.__analyseGadgetsProgress__
+        for i,g in enumerate(gadgets):
+            g.info = analyser.analyse(g)
+            if cb:
+                cb(g, float(i)/lg)
+        if cb:
+             cb(None, 1.0)
+        self.__saveCache(fileObject)
+        fileObject.analysed = True
+
     def loadGadgetsFor(self, name=None):
+
         def load_gadgets(f):
             gtype = None
             cache = False
-            Gadget.IMAGE_BASES[f.name] = f.loader.imageBase
+            Gadget.IMAGE_BASES[f.loader.checksum] = f.loader.imageBase
             if self.options.type == 'rop':
                 gtype = GadgetType.ROP
             elif self.options.type == 'jop':
@@ -572,15 +609,19 @@ class RopperService(object):
             elif self.options.type == 'sys':
                 gtype = GadgetType.SYS
             elif self.options.type == 'all':
-                gtype = GadgetType.ALL    
+                gtype = GadgetType.ALL
             f.allGadgets = self.__loadCache(f)
             if f.allGadgets == None:
                 cache = True
                 f.allGadgets = self.__ropper.searchGadgets(f.loader, instructionCount=self.options.inst_count, gtype=gtype)
+
             if cache:
                 self.__saveCache(f)
             f.gadgets = self.__prepareGadgets(f, f.allGadgets, f.type)
-         
+            f.analysed = f.gadgets[0].info is not None if len(f.gadgets) > 0 else False
+            #self._analyseGadgets(f.gadgets)
+
+
         if name is None:
             for fc in self.__files:
                 load_gadgets(fc)
@@ -588,7 +629,7 @@ class RopperService(object):
             for fc in self.__files:
                 if fc.loader.fileName == name:
                     load_gadgets(fc)
-                
+
     def printGadgetsFor(self, name=None):
         def print_gadgets(f):
             print(f.loader.fileName)
@@ -640,16 +681,42 @@ class RopperService(object):
             fc = self._getFileFor(name)
             if not fc:
                 raise RopperError('No such file opened: %s' % name)
-            
+
             s = fc.loader.arch.searcher
             for gadget in s.search(fc.gadgets, search, quality):
                     yield(fc.name, gadget)
-        else:        
+        else:
             for fc in self.__files:
                 s = fc.loader.arch.searcher
                 for gadget in s.search(fc.gadgets, search, quality):
                     yield(fc.name, gadget)
 
+    def semanticSearch(self, search, stableRegs=[], name=None):
+        count = 0
+        if name:
+            fc = self._getFileFor(name)
+            if not fc:
+                raise RopperError('No such file opened: %s' % name)
+
+            s = fc.loader.arch.searcher
+            for gadget in s.semanticSearch(fc.gadgets, search, self.options.inst_count, stableRegs):
+                if self.options.count_of_findings == 0 or self.options.count_of_findings > count:
+                    yield(fc.name, gadget)
+                else:
+                    break
+                count += 1
+            self.__saveCache(fc)
+        else:
+            for fc in self.__files:
+                s = fc.loader.arch.searcher
+                for gadget in s.semanticSearch(fc.gadgets, search, self.options.inst_count, stableRegs):
+                    if self.options.count_of_findings == 0 or self.options.count_of_findings > count:
+                        yield(fc.name, gadget)
+                    else:
+                        break
+                    count += 1
+                self.__saveCache(fc)
+
     def searchdict(self, search, quality=None, name=None):
         to_return = {}
         for file, gadget in self.search(search, quality, name):
@@ -674,13 +741,13 @@ class RopperService(object):
                 g = ropper.disassembleAddress(section, fc.loader, address, address - (fc.loader.imageBase+section.offset), length)
                 if not g:
                     raise RopperError('Cannot disassemble address: %s' % toHex(address))
-                    
+
                 if length < 0:
                     length = length * -1
                 return g.disassemblyString()
         return ''
-        
-    def createRopChain(self, chain, options={}):
+
+    def createRopChain(self, chain, arch, options={}):
         callback = None
         if self.__callbacks and hasattr(self.__callbacks, '__ropchainMessages__'):
             callback = self.__callbacks.__ropchainMessages__
@@ -688,8 +755,9 @@ class RopperService(object):
         b = []
         gadgets = {}
         for binary in self.__files:
-            gadgets[binary.loader] = binary.gadgets
-            b.append(binary.loader)
+            if str(binary.arch) == arch:
+                gadgets[binary.loader] = binary.gadgets
+                b.append(binary.loader)
         generator = RopChain.get(b, gadgets, chain, callback, unhexlify(self.options.badbytes))
 
         if not generator:
@@ -702,7 +770,7 @@ class RopperService(object):
         if not file:
             raise RopperError('No such file opened: %s' % name)
         file.loader.imageBase = imagebase
-        Gadget.IMAGE_BASES[name] = file.loader.imageBase
+        Gadget.IMAGE_BASES[file.loader.checksum] = file.loader.imageBase
         if file.loaded and (self.options.badbytes or self.options.cfg_only and file.type == Type.PE):
             file.gadgets = self.__prepareGadgets(file, file.allGadgets, file.type)
 
@@ -736,7 +804,7 @@ class FileContainer(object):
     @property
     def name(self):
         return self.loader.fileName
-    
+
     @property
     def arch(self):
         return self.loader.arch
@@ -744,7 +812,7 @@ class FileContainer(object):
     @property
     def type(self):
         return self.loader.type
-    
+
     @property
     def loaded(self):
         return self.__loaded
@@ -752,7 +820,7 @@ class FileContainer(object):
     @property
     def loader(self):
         return self.__loader
-    
+
     @property
     def gadgets(self):
         return self.__gadgets
@@ -760,7 +828,7 @@ class FileContainer(object):
     @gadgets.setter
     def gadgets(self, gadgets):
         self.__gadgets = gadgets
-    
+
     @property
     def allGadgets(self):
         return self.__all_gadgets
@@ -768,4 +836,4 @@ class FileContainer(object):
     @allGadgets.setter
     def allGadgets(self, gadgets):
         self.__loaded = True if gadgets is not None else False
-        self.__all_gadgets = gadgets
\ No newline at end of file
+        self.__all_gadgets = gadgets
